﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace DxPdfApp;
using namespace Platform;
using namespace concurrency;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Graphics::Display;
using namespace Windows::Storage;
using namespace Windows::Storage::Pickers;
using namespace Windows::Data::Pdf;
using namespace Microsoft::WRL;


// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238


MainPage::MainPage() :
m_currentPageIndex(UINT_MAX)
{
	InitializeComponent();
	
	auto info = DisplayInformation::GetForCurrentView();

	m_renderer = ref new Renderer(
		Window::Current->Bounds,
		info->LogicalDpi
		);
	m_spPageImageSource = ref new PageImageSource(m_renderer);

	PageImage->Source = m_spPageImageSource->ImageSource;
}

void MainPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	(void)e;    // Unused parameter  
}  

void MainPage::Button_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	FileOpenPicker^ openPicker = ref new FileOpenPicker();
	openPicker->SuggestedStartLocation = PickerLocationId::DocumentsLibrary;
	openPicker->ViewMode = PickerViewMode::List;
	openPicker->FileTypeFilter->Clear();
	openPicker->FileTypeFilter->Append(L".pdf");

	create_task(openPicker->PickSingleFileAsync()).then([this](StorageFile^ pdfFile) {
		create_task(PdfDocument::LoadFromFileAsync(pdfFile)).then([this](PdfDocument^ pdfDoc){
			m_pdfDoc = pdfDoc;
			GoToPage(0);
		});
	});
}

	

void MainPage::ScaleValueChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs ^ e)
{
	SetPageImageSize();
	UpdatePageImage();
}

void MainPage::SetCurrentPage(Windows::Data::Pdf::PdfPage^ pdfPage)
{
	m_pdfPage = pdfPage;
	m_spPageImageSource->SetPage(m_pdfPage);
}

void MainPage::SetPageImageSize()
{
	Size pageSize = m_pdfPage->Size;
	double width = pageSize.Width;
	double height = pageSize.Height;		
	PageImage->Width = width;
	PageImage->Height = height;
}

void MainPage::UpdatePageImage()
{
	double width = PageImage->Width;
	double height = PageImage->Height;
	HRESULT hr = m_spPageImageSource->Resize(width, height);
	if (FAILED(hr))
	{
		PageImage->Source = nullptr;
		m_spPageImageSource = nullptr;

		m_spPageImageSource = ref new PageImageSource(m_renderer);
		PageImage->Source = m_spPageImageSource->ImageSource;
		m_spPageImageSource->SetPage(m_pdfPage);
		m_spPageImageSource->Resize(width, height);
	}
}

void MainPage::GoToPage(unsigned int index)
{
	m_currentPageIndex = index;
	auto pdfPage = m_pdfDoc->GetPage(m_currentPageIndex);
	Size pageSize = pdfPage->Size;
	SetCurrentPage(pdfPage);
	SetPageImageSize();
	UpdatePageImage();
}
