#pragma once

namespace DxPdfApp
{
	ref class PageImageSource sealed
	{
	public:
		PageImageSource(Renderer^ renderer, bool fUseVSIS = true);
		void SetPage(Windows::Data::Pdf::PdfPage^ pdfPage);
		void CancelPageUpdates();
		property Windows::UI::Xaml::Media::Imaging::VirtualSurfaceImageSource^ ImageSource
		{
			Windows::UI::Xaml::Media::Imaging::VirtualSurfaceImageSource^ get()
			{
				return m_vsis;
			}
		}
		void UpdatesNeeded();

	internal:
		HRESULT Resize(double width, double height);

	private:
		~PageImageSource();
		void RenderPageRect(RECT rect);

	private:
		Renderer^                                                       m_renderer;
		Windows::Data::Pdf::PdfPage^                                    m_pdfPage;
		concurrency::task<void>                                         m_spRenderTask;
		concurrency::cancellation_token_source                          m_cts;

		double                                                          m_width, m_height;

		Windows::UI::Xaml::Media::Imaging::VirtualSurfaceImageSource^   m_vsis;
		Microsoft::WRL::ComPtr<IVirtualSurfaceImageSourceNative>        m_vsisNative;
	};

	struct VSISData
	{
		Microsoft::WRL::ComPtr<IDXGISurface> dxgiSurface;
		POINT offset;
		bool fContinue;

		VSISData() { offset.x = offset.y = 0; fContinue = false; }
	};

	class VSISCallBack : public Microsoft::WRL::RuntimeClass<Microsoft::WRL::RuntimeClassFlags<Microsoft::WRL::ClassicCom>,
		IVirtualSurfaceUpdatesCallbackNative>
	{
	public:
		VSISCallBack(Platform::WeakReference wpPageImageSource) : m_wpPageImageSource(wpPageImageSource) { }

		// IVirtualSurfaceUpdatesCallbackNative          
		IFACEMETHOD (UpdatesNeeded)()  
		{
			PageImageSource^ spPageImageSource = m_wpPageImageSource.Resolve<PageImageSource>();
			if (spPageImageSource != nullptr)
			{
				spPageImageSource->UpdatesNeeded();
			}
			return S_OK;
		}

	private:
		Platform::WeakReference        m_wpPageImageSource;
	};
}