﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"
#include <ppltasks.h>

using namespace PdfAPISample;

using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Media::Imaging;
using namespace Windows::UI::Xaml::Navigation;
using namespace Windows::Storage;
using namespace Windows::Storage::Pickers;
using namespace Windows::Storage::Streams;
using namespace Windows::Data::Pdf;
using namespace concurrency;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

MainPage::MainPage()
{
	InitializeComponent();
	m_streamVec = ref new Vector<PdfPageAsImage^>();
}

void MainPage::Button_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args)
{
	m_streamVec->Clear();

	FileOpenPicker^ openPicker = ref new FileOpenPicker();
	openPicker->SuggestedStartLocation = PickerLocationId::DocumentsLibrary;
	openPicker->ViewMode = PickerViewMode::List;
	openPicker->FileTypeFilter->Clear();
	openPicker->FileTypeFilter->Append(L".pdf");

	create_task(openPicker->PickSingleFileAsync()).then([this](StorageFile^ pdfFile)
	{
		m_ImagefileName = pdfFile->Name;
		create_task(PdfDocument::LoadFromFileAsync(pdfFile)).then([this](PdfDocument^ pdfDoc)
		{
			auto page = pdfDoc->GetPage(0);
			auto stream = ref new InMemoryRandomAccessStream();
			IAsyncAction^ action = page->RenderToStreamAsync(stream);
			auto actionTask = create_task(action);
			actionTask.then([this, stream, page]()
			{
				String^ img_name = m_ImagefileName + ".png";
				task<StorageFolder^> writeFolder(KnownFolders::PicturesLibrary->GetFolderAsync("Output"));
				writeFolder.then([this, img_name, stream, page](StorageFolder^ outputFolder) {
					task<StorageFile^> file(outputFolder->CreateFileAsync(img_name,
						CreationCollisionOption::ReplaceExisting));
					file.then([this, stream, page](StorageFile^ file1) {
						task<IRandomAccessStream^> writeStream(file1->OpenAsync(FileAccessMode::ReadWrite));
						writeStream.then([this, stream, page](IRandomAccessStream^ fileStream) {
							IAsyncOperationWithProgress<unsigned long long, unsigned long long>^ progress =
								RandomAccessStream::CopyAndCloseAsync(stream->GetInputStreamAt(0), fileStream->GetOutputStreamAt(0));
							auto copyTask = create_task(progress);
							copyTask.then([this, stream, page, fileStream](unsigned long long bytesWritten) {
								stream->Seek(0);
								auto bmp = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage();
								bmp->SetSource(fileStream);
								auto page1 = ref new PdfPageAsImage();
								page1->PdfPageImage = bmp;
								m_streamVec->Append(page1);
								pageView->ItemsSource = ImageCollection;
								delete stream;
								delete page;
							});
						});
					});
				});
			});
		});
	});
}

void MainPage::OptionsButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args)
{
	m_streamVec->Clear();
	FileOpenPicker^ openPicker = ref new FileOpenPicker();
	openPicker->SuggestedStartLocation = PickerLocationId::DocumentsLibrary;
	openPicker->ViewMode = PickerViewMode::List;
	openPicker->FileTypeFilter->Clear();
	openPicker->FileTypeFilter->Append(L".pdf");

	create_task(openPicker->PickSingleFileAsync()).then([this](StorageFile^ pdfFile)
	{
		m_ImagefileName = pdfFile->Name;
		create_task(PdfDocument::LoadFromFileAsync(pdfFile)).then([this](PdfDocument^ pdfDoc)
		{
			auto page = pdfDoc->GetPage(0);
			auto stream = ref new InMemoryRandomAccessStream();
			PdfPageRenderOptions options;
			options.BackgroundColor = Windows::UI::Colors::Azure;
			options.BitmapEncoderId = Windows::Graphics::Imaging::BitmapEncoder::JpegEncoderId;


			IAsyncAction^ action = page->RenderToStreamAsync(stream, %options);
			auto actionTask = create_task(action);
			actionTask.then([this, stream, page]()
			{
				String^ img_name = m_ImagefileName + ".jpg";
				task<StorageFolder^> writeFolder(KnownFolders::PicturesLibrary->GetFolderAsync("Output"));
				writeFolder.then([this, img_name, stream, page](StorageFolder^ outputFolder) {
					task<StorageFile^> file(outputFolder->CreateFileAsync(img_name, CreationCollisionOption::ReplaceExisting));
					file.then([this, stream, page](StorageFile^ file1) {
						task<IRandomAccessStream^> writeStream(file1->OpenAsync(FileAccessMode::ReadWrite));
						writeStream.then([this, stream, page](IRandomAccessStream^ fileStream) {
							IAsyncOperationWithProgress<unsigned long long, unsigned long long>^ progress = RandomAccessStream::CopyAndCloseAsync(stream->GetInputStreamAt(0), fileStream->GetOutputStreamAt(0));
							auto copyTask = create_task(progress);
							copyTask.then([this, stream, page, fileStream](unsigned long long bytesWritten) {
								stream->Seek(0);
								auto bmp = ref new Windows::UI::Xaml::Media::Imaging::BitmapImage();
								bmp->SetSource(fileStream);
								auto page1 = ref new PdfPageAsImage();
								page1->PdfPageImage = bmp;
								m_streamVec->Append(page1);
								pageView->ItemsSource = ImageCollection;
								delete stream;
								delete page;
							});
						});
					});
				});
			});
		});
	});
}